# http2-wrapper (under heavy development)
> Use HTTP2 the same way like HTTP1

[![Build Status](https://travis-ci.org/szmarczak/http2-wrapper.svg?branch=master)](https://travis-ci.org/szmarczak/http2-wrapper) [![Coverage Status](https://coveralls.io/repos/github/szmarczak/http2-wrapper/badge.svg?branch=master)](https://coveralls.io/github/szmarczak/http2-wrapper?branch=master)
![npm](https://img.shields.io/npm/dm/http2-wrapper.svg)
 [![install size](https://packagephobia.now.sh/badge?p=http2-wrapper@0.1.1)](https://packagephobia.now.sh/result?p=http2-wrapper@0.1.1)

## Usage
```js
'use strict';
const http2 = require('http2-wrapper');

const options = {
	hostname: 'nghttp2.org',
	protocol: 'https:',
	path: '/httpbin/post',
	method: 'POST',
	headers: {
		'content-length': 6
	}
};

const req = http2.request(options, res => {
	console.log('statusCode:', res.statusCode);
	console.log('headers:', res.headers);

	const body = [];
	res.on('data', chunk => {
		body.push(chunk);
	});
	res.on('end', () => {
		console.log('body:', Buffer.concat(body).toString());
	});
});

req.on('error', e => console.error(e));

req.write('123');
req.end('456');

// statusCode: 200
// headers: { ':status': 200,
//   date: 'Sat, 11 Aug 2018 09:37:41 GMT',
//   'content-type': 'application/json',
//   'content-length': '264',
//   'access-control-allow-origin': '*',
//   'access-control-allow-credentials': 'true',
//   'x-backend-header-rtt': '0.002997',
//   'strict-transport-security': 'max-age=31536000',
//   server: 'nghttpx',
//   via: '1.1 nghttpx',
//   'x-frame-options': 'SAMEORIGIN',
//   'x-xss-protection': '1; mode=block',
//   'x-content-type-options': 'nosniff' }
// body: {
//   "args": {},
//   "data": "123456",
//   "files": {},
//   "form": {},
//   "headers": {
//     "Content-Length": "6",
//     "Host": "nghttp2.org:443",
//     "Via": "2 nghttpx"
//   },
//   "json": 123456,
//   "origin": "xxx.xxx.xxx.xxx",
//   "url": "https://nghttp2.org:443/httpbin/post"
// }
```

## TODO

### HTTP2IncomingMessage

 * [x] Event: 'aborted'
 * [x] Event: 'close'
 * [x] message.destroy([error])
 * [x] message.headers
 * [x] message.httpVersion
 * [x] message.rawHeaders
 * [x] message.rawTrailers
 * [x] message.setTimeout(msecs, callback)
 * [x] message.socket
 * [x] message.statusCode
 * [x] message.statusMessage
 * [x] message.trailers

### HTTP2ClientRequest
 * [x] Event: 'abort'
 * [ ] Event: 'connect'
 * [x] Event: 'continue'
 * [ ] ~~Event: 'information'~~
 * [x] Event: 'response'
 * [x] Event: 'socket'
 * [x] Event: 'timeout'
 * [ ] Event: 'upgrade'
 * [x] request.abort()
 * [x] request.aborted
 * [x] request.connection
 * [x] request.end([data[, encoding]][, callback])
 * [x] request.flushHeaders()
 * [x] request.getHeader(name)
 * [ ] ~~request.maxHeadersCount~~
 * [x] request.removeHeader(name)
 * [x] request.setHeader(name, value)
 * [x] request.setNoDelay([noDelay])
 * [ ] ~~request.setSocketKeepAlive([enable][, initialDelay])~~
 * [x] request.setTimeout(timeout[, callback])
 * [x] request.socket
 * [x] request.write(chunk[, encoding][, callback])
