'use strict';
const {Readable} = require('stream');

const kDidRead = Symbol('didRead');

class HTTP2IncomingMessage extends Readable {
	constructor(req, response) {
		super();

		this.statusCode = response.headers[':status'];
		this.statusMessage = '';
		this.httpVersion = '2.0';
		this.httpVersionMajor = 2;
		this.httpVersionMinor = 0;
		this.headers = response.headers;
		this.trailers = [];
		this.req = req;

		this.aborted = false;
		this.complete = false;

		this.rawHeaders = response.rawHeaders;
		this.rawTrailers = [];

		this.socket = req.socket;
		this.connection = req.socket;

		const self = this;

		this.on('end', () => {
			if (!self.req.aborted) {
				self.complete = true;
			}
		});

		this[kDidRead] = false;
		req._request.on('end', () => self.push(null));

		this._dumped = false;
	}

	_read() {
		if (this[kDidRead]) {
			process.nextTick(stream => stream.resume(), this.req._request);
		} else {
			this[kDidRead] = true;
			const self = this;
			const listener = chunk => {
				if (!self.push(chunk)) {
					self.req._request.pause();
				}
			};
			this.req._request.on('data', listener);
		}
	}

	destroy(error) {
		this.req._request.destroy(error);
	}

	setTimeout(ms, callback) {
		this.req.setTimeout(ms, callback);
		return this;
	}

	_dump() {
		if (!this._dumped) {
			this._dumped = true;
			this.req._request.removeAllListeners('data');
			this.req._request.resume();

			this.removeAllListeners('data');
			this.resume();
		}
	}
}

module.exports = HTTP2IncomingMessage;
