'use strict';
/* istanbul ignore file */

module.exports = {};

const makeError = (Base, key, getMessage) => {
	module.exports[key] = class NodeError extends Base {
		constructor(...args) {
			super(typeof getMessage === 'string' ? getMessage : getMessage(args));
		}

		get name() {
			return `${super.name} [${key}]`;
		}

		set name(value) {
			Object.defineProperty(this, 'name', {
				configurable: true,
				enumerable: true,
				value,
				writable: true
			});
		}

		get code() {
			return key;
		}

		set code(value) {
			Object.defineProperty(this, 'code', {
				configurable: true,
				enumerable: true,
				value,
				writable: true
			});
		}
	};
};

makeError(Error, 'ERR_HTTP2_HEADERS_SENT', args => {
	if (args && args[0]) {
		return `Cannot ${args[0]} headers after they are sent to the client`;
	}

	return 'Response has already been initated.';
});

makeError(TypeError, 'ERR_HTTP2_PSEUDOHEADER_NOT_ALLOWED', 'Cannot set HTTP/2 pseudo-headers');

makeError(TypeError, 'ERR_INVALID_HTTP_TOKEN', args => {
	return `${args[0]} must be a valid HTTP token ["${args[1]}"]`;
});

makeError(TypeError, 'ERR_HTTP2_INVALID_HEADER_VALUE', args => {
	return `Invalid value "${args[0]}" for header "${args[1]}"`;
});

module.exports.createHangUpError = () => {
	const err = new Error('socket hang up');
	err.code = 'ECONNRESET';
	return err;
};
